#!/usr/bin/env python3
"""
TEST.EXTENSIONS.EXTENSIONS_TESTMOD.PY -- utility module for Python/C extension tests
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module provides a Python interface to the C code
that is needed to test PLIB3.EXTENSIONS's Python/C extensions.
"""

from ._extensions_testmod import *



#!/usr/bin/env python3
"""
Sub-Package EXTENSIONS of Package PLIB3 -- Python/C Extensions
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains extensions written using the
Python/C API. Currently the following extensions are
implemented:

function ``capsule_compare``: takes two Capsules and returns a
    ``bool`` indicating whether they wrap the same C-level pointer.
"""

from ._extensions import *



#!/usr/bin/python3 -u
"""
Setup script for PLIB3.EXTENSIONS package
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""


version = "1.1.post1"

name = "plib3.extensions"
description = "Useful Python 3 Python/C API functions."

author = "Peter A. Donis"
author_email = "peterdonis@alum.mit.edu"

startline = 5
endspec = "The Zen of PLIB3"

dev_status = "Production"

license = "GPLv2"

ext_names = [
    'plib.extensions._extensions',
    'plib.test.extensions._extensions_testmod'
]
ext_srcdir = "src"

classifiers = """
Environment :: Console
Environment :: MacOS X
Environment :: Win32 (MS Windows)
Intended Audience :: Developers
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: POSIX :: Linux
Topic :: Software Development :: Libraries :: Python Modules
"""

requires = """
plib3.stdlib (>=1.1)
"""

rst_header_template = """**{basename}** for {name} {version}

:Author:        {author}
:Release Date:  {releasedate}
"""


if __name__ == '__main__':
    import sys
    import os
    from subprocess import call
    from distutils.core import setup
    from setuputils import convert_rst, current_date, setup_vars, long_description as make_long_description
    
    if "sdist" in sys.argv:
        convert_rst(rst_header_template,
            startline=2,
            name=name.upper(),
            version=version,
            author=author,
            releasedate=current_date("%d %b %Y")
        )
        call(['sed', '-i', 's/gitlab.com\/pdonis\/plib-/pypi.org\/project\/plib./', 'README'])
        call(['sed', '-i', 's/gitlab.com\/pdonis\/plib3-/pypi.org\/project\/plib3./', 'README'])
        call(['sed', '-i', 's/gitlab.com\/pdonis/pypi.org\/project/', 'README'])
    elif os.path.isfile("README.rst"):
        startline = 3
        long_description = make_long_description(globals(), filename="README.rst")
    setup(**setup_vars(globals()))



#!/usr/bin/env python3
"""
TEST.EXTENSIONS.TEST_EXTENSIONS.PY -- test script for Python/C extensions
Copyright (C) 2008-2019 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the Python/C extensions in
the PLIB3.EXTENSIONS sub-package.
"""

import unittest

from plib.stdlib.iters import split_n
from plib.extensions import capsule_compare

from plib.test.extensions import extensions_testmod


class Test_capsule_compare(unittest.TestCase):
    
    def test_compare(self):
        o_orig = extensions_testmod.capsule_orig()
        o_same = extensions_testmod.capsule_same()
        o_diff = extensions_testmod.capsule_different()
        
        n_orig = extensions_testmod.capsule_null_orig()
        n_same = extensions_testmod.capsule_null_same()
        n_diff = extensions_testmod.capsule_null_different()
        
        capsules = [o_orig, o_same, o_diff, n_orig, n_same, n_diff]
        for i, capsule in enumerate(capsules):
            for other in capsules[:i] + capsules[i+1:]:
                self.assertTrue(capsule is not other)
        
        self.assertTrue(all(capsule_compare(c1, c2) == capsule_compare(c2, c1)
                        for c1 in capsules for c2 in capsules))
        
        o_capsules, n_capsules = split_n(3, capsules)
        
        self.assertTrue(all(capsule_compare(o, o) for o in o_capsules))
        self.assertTrue(capsule_compare(o_orig, o_same))
        self.assertFalse(capsule_compare(o_orig, o_diff))
        
        self.assertTrue(all(capsule_compare(n, n) for n in n_capsules))
        self.assertTrue(capsule_compare(n_orig, n_same))
        self.assertFalse(capsule_compare(n_orig, n_diff))
        
        self.assertTrue(all(capsule_compare(o, n) for o, n in zip(o_capsules, n_capsules)))
        self.assertTrue(capsule_compare(o_orig, n_same))
        self.assertFalse(capsule_compare(o_orig, n_diff))
        self.assertTrue(capsule_compare(n_orig, o_same))
        self.assertFalse(capsule_compare(n_orig, o_diff))
        
        self.assertRaises(TypeError, capsule_compare)
        self.assertRaises(TypeError, capsule_compare, None)
        self.assertRaises(TypeError, capsule_compare, None, None)
        self.assertRaises(TypeError, capsule_compare, None, o_orig)
        self.assertRaises(TypeError, capsule_compare, o_orig, None)
        self.assertRaises(TypeError, capsule_compare, None, None, None)


if __name__ == '__main__':
    unittest.main()
